<?php

namespace Drupal\ump\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\UserInterface;

/**
 * UMP access controller.
 */
class AccessController {

  /**
   * Checks access for a user create.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Current user.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function create(AccountInterface $account) {
    if (ump_check_permission('create')) {
      return AccessResult::allowed();
    }
    return AccessResult::allowedIfHasPermission($account, 'administer users');
  }

  /**
   * Checks access for a user view.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Current user.
   * @param \Drupal\user\UserInterface $user
   *   Run access checks against this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function view(AccountInterface $account, UserInterface $user) {
    if (ump_check_permission('view', $user)) {
      return AccessResult::allowed();
    }
    return $user->access('view', NULL, TRUE);
  }

  /**
   * Checks access for a user edit.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Current user.
   * @param \Drupal\user\UserInterface $user
   *   Run access checks against this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function edit(AccountInterface $account, UserInterface $user) {
    if (ump_check_permission('edit', $user)) {
      return AccessResult::allowed();
    }
    return $user->access('update', NULL, TRUE);
  }

  /**
   * Checks access for a user delete.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Current user.
   * @param \Drupal\user\UserInterface $user
   *   Run access checks against this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function delete(AccountInterface $account, UserInterface $user) {
    if (ump_check_permission('delete', $user)) {
      return AccessResult::allowed();
    }
    return $user->access('delete', NULL, TRUE);
  }

  /**
   * Checks access for a user create.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Current user.
   * @param \Drupal\user\UserInterface $user
   *   Run access checks against this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function userUmpTab(AccountInterface $account, UserInterface $user) {
    if ($account->id() == $user->id() && ump_check_permission('ump_tab')) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

}
