<?php

/**
 * @file
 * Helper functions for UMP module.
 */

use Drupal\user\Entity\User;

/**
 * Helper function to get the UMP rules defined for the current user.
 */
function ump_get_ump_crud_rules() {
  $user = User::load(\Drupal::currentUser()->id());
  $return_rules = [];
  $rules = ump_get_ump_crud_rules_all();
  if (is_array($rules)) {
    foreach ($rules as $rule_key) {
      $rule = \Drupal::config('ump.settings')->get($rule_key);
      if (in_array($rule["select_role"], $user->getRoles())) {
        $return_rules[] = $rule;
      }
    }
  }
  return $return_rules;
}

/**
 * Helper function to return array of defined UMP permissions.
 */
function ump_get_ump_permissions() {
  $permission_array = [];
  $rules = ump_get_ump_crud_rules_all();
  foreach ($rules as $rule_key) {
    $rule = \Drupal::config('ump.settings')->get($rule_key);

    if ($rule["allow_to_create_enable"] == 1) {
      $permission_array[$rule["select_role"]]["create"][$rule["allow_to_create_role"]] = 1;
      if ($rule["allow_to_create_overtake"] == 1) {
        $permission_array[$rule["select_role"]]["create"][$rule["allow_to_create_role"]] = ["overtake" => $rule["allow_to_create_overtake_field"]];
      }
    }
    if ($rule["allow_to_view_enable"] == 1) {
      $permission_array[$rule["select_role"]]["view"][$rule["allow_to_view_role"]]["which"] = $rule["allow_to_view_which"];
      if ($rule["allow_to_view_which"] == "if_share_common_field") {
        $permission_array[$rule["select_role"]]["view"][$rule["allow_to_view_role"]]["common_field"] = $rule["allow_to_view_common_field"];
      }
    }
    if ($rule["allow_to_edit_enable"] == 1) {
      $permission_array[$rule["select_role"]]["edit"][$rule["allow_to_edit_role"]]["which"] = $rule["allow_to_edit_which"];
      if ($rule["allow_to_edit_which"] == "if_share_common_field") {
        $permission_array[$rule["select_role"]]["edit"][$rule["allow_to_edit_role"]]["common_field"] = $rule["allow_to_edit_common_field"];
      }
    }
    if ($rule["allow_to_delete_enable"] == 1) {
      $permission_array[$rule["select_role"]]["delete"][$rule["allow_to_delete_role"]]["which"] = $rule["allow_to_delete_which"];
      if ($rule["allow_to_delete_which"] == "if_share_common_field") {
        $permission_array[$rule["select_role"]]["delete"][$rule["allow_to_delete_role"]]["common_field"] = $rule["allow_to_delete_common_field"];
      }
    }
  }

  return $permission_array;
}

/**
 * Helper function to return creator UID.
 */
function ump_get_ump_user_created_by_uid($uid) {
  $load_user = User::load($uid);
  if ($load_user) {
    return $load_user->field_ump_created_by->value;
  }
  return NULL;
}

/**
 * Helper function to get the all UMP rules defined.
 */
function ump_get_ump_crud_rules_all() {
  $conf = \Drupal::config('ump.settings')->get();
  $rules = [];
  foreach ($conf as $key => $value) {
    if (substr($key, 0, 14) === "ump_CRUD_rule_") {
      $rules[] = $key;
    }
  }
  return $rules;
}

/**
 * Helper function to get the all user roles defined.
 */
function ump_get_user_roles() {
  $get_roles = [];
  $user_roles = user_roles(TRUE);
  foreach ($user_roles as $key => $role) {
    $get_roles[$key] = $role->label();
  }
  return $get_roles;
}

/**
 * Helper function to get all user fields defined.
 */
function ump_get_user_user_fields() {
  // TODO: get the label of the fields.
  $user_fields = [];
  // Define the fields which we don't want to show.
  $fields_to_skip = ['field_ump_created_by'];

  $system_fields = Drupal::service('entity_field.manager')->getFieldDefinitions('user', '');
  $all_fields = Drupal::service('entity_field.manager')->getFieldStorageDefinitions('user');
  $diff_keys = array_diff(array_keys($all_fields), array_keys($system_fields));

  foreach ($diff_keys as $key) {
    if (!in_array($key, $fields_to_skip)) {
      $user_fields[$key] = $key;
    }
  }
  return $user_fields;
}

/**
 * Callback function for permission check.
 */
function ump_check_permission($action, $load_user = FALSE) {
  $user = User::load(\Drupal::currentUser()->id());

  if (in_array("administrator", $user->getRoles())) {
    return TRUE;
  }

  $ump_permissions = ump_get_ump_permissions();

  if ($action == "create") {
    foreach ($user->getRoles() as $role_value) {
      if (isset($ump_permissions[$role_value]["create"])) {
        return TRUE;
      }
    }
  }

  elseif ($action == "edit") {
    if ($load_user) {
      foreach ($user->getRoles() as $role_value) {
        foreach ($load_user->getRoles() as $load_user_value) {
          if (isset($ump_permissions[$role_value]["edit"][$load_user_value]['which'])) {
            if ($ump_permissions[$role_value]["edit"][$load_user_value]['which'] == 'all') {
              return TRUE;
            }
            elseif ($ump_permissions[$role_value]["edit"][$load_user_value]['which'] == 'created_by_user') {
              if (ump_get_ump_user_created_by_uid($load_user->id()) == $user->id()) {
                return TRUE;
              }
            }
            elseif ($ump_permissions[$role_value]["edit"][$load_user_value]['which'] == 'if_share_common_field' && isset($ump_permissions[$role_value]["edit"][$load_user_value]['common_field'])) {
              $common_field = $ump_permissions[$role_value]["edit"][$load_user_value]['common_field'];
              if (serialize($load_user->$common_field->getValue()) == serialize($user->$common_field->getValue())) {
                return TRUE;
              }
            }
          }
        }
      }
    }
  }

  elseif ($action == "view") {
    if ($load_user) {
      foreach ($user->getRoles() as $role_value) {
        foreach ($load_user->getRoles() as $load_user_value) {
          if (isset($ump_permissions[$role_value]["view"][$load_user_value]['which'])) {
            if ($ump_permissions[$role_value]["view"][$load_user_value]['which'] == "all") {
              return TRUE;
            }
            elseif ($ump_permissions[$role_value]["view"][$load_user_value]['which'] == "created_by_user") {
              if (ump_get_ump_user_created_by_uid($load_user->id()) == $user->id()) {
                return TRUE;
              }
            }
            elseif ($ump_permissions[$role_value]["view"][$load_user_value]['which'] == 'if_share_common_field' && isset($ump_permissions[$role_value]["view"][$load_user_value]['common_field'])) {
              $common_field = $ump_permissions[$role_value]["view"][$load_user_value]['common_field'];
              if (serialize($load_user->$common_field->getValue()) == serialize($user->$common_field->getValue())) {
                return TRUE;
              }
            }
          }
        }
      }
    }
  }

  elseif ($action == "delete") {
    if ($load_user) {
      foreach ($user->getRoles() as $role_value) {
        foreach ($load_user->getRoles() as $load_user_value) {
          if (isset($ump_permissions[$role_value]["delete"][$load_user_value]['which'])) {
            if ($ump_permissions[$role_value]["delete"][$load_user_value]['which'] == "all") {
              return TRUE;
            }
            elseif ($ump_permissions[$role_value]["delete"][$load_user_value]['which'] == "created_by_user") {
              if (ump_get_ump_user_created_by_uid($load_user->id()) == $user->id()) {
                return TRUE;
              }
            }
            elseif ($ump_permissions[$role_value]["delete"][$load_user_value]['which'] == 'if_share_common_field' && isset($ump_permissions[$role_value]["delete"][$load_user_value]['common_field'])) {
              $common_field = $ump_permissions[$role_value]["delete"][$load_user_value]['common_field'];
              if (serialize($load_user->$common_field->getValue()) == serialize($user->$common_field->getValue())) {
                return TRUE;
              }
            }
          }
        }
      }
    }
  }
  
  elseif ($action == "ump_tab") {
    foreach ($user->getRoles() as $role_value) {
      if (isset($ump_permissions[$role_value])) {
        return TRUE;
      }
    }
  }

  return FALSE;
}
